 
<BLOCK>
<LICENSE>
/*  
    Capsim (r) Text Mode Kernel (TMK) 
    Copyright (C) 1989-2017  Silicon DSP  Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.silicondsp.com
    Silicon DSP  Corporation
    Las Vegas, Nevada
*/

</LICENSE>
<BLOCK_NAME>eigen</BLOCK_NAME> 



<COMMENTS>
<![CDATA[ 
/*
 * 		eigen()
 *
 * Generated by stargen
 *
 * Image Termination Star
 * Notes:
 *    This is a star generated from the image terminator template
 *    It will input a sample.
 * 
 *    
 * Programmer: SHA
 * Date: March 10, 2003
 * Modified: 
 *
 */
 
]]>
</COMMENTS> 
<DESC_SHORT>
  A eignevalue problem is solved: Ax=lambda*x
  where A is a real MxM matrix.
</DESC_SHORT>
<DESCRIPTION>
 /*
 * This program demonstrates the CLAPACK library
 *
 * A eignevalue problem is solved: Ax=lambda*x
 * where A is a real MxM matrix.
 *
 * The prgram will compute the eigenvalues and the eigenvectors using the
 * CLAPACK routine : dgeev (double precision general matrix eigen value)
 *
 * The program sets up a matrix using a C two dimensional array.
 * The array is then copied into a vector form suitable for calls into the
 * CLAPACK library
 *
 *
 * Note this program is an illustration of how to use CLAPACK and is not efficient
 * in storage etc. The idea is to show how to use CLAPACK and convert between the f2c 
 * two dimensional form ( essentially a single dimensional array) and the more 
 * usefull C two dimensional array.
 *
 * The image probe provides data in the C two dimensional form a[][].
 * So to use CLAPACK, the matrix  must be mapped to a single dimensional array.
 *
 */ 
</DESCRIPTION>
<PROGRAMMERS>
Sasan Ardalan
</PROGRAMMERS>

<INCLUDES>
<![CDATA[ 
#include <f2c.h>
#include <clapack.h>
]]>
</INCLUDES>

     

<PARAMETERS>
<PARAM>
	<DEF>File to store results</DEF>
	<TYPE>file</TYPE>
	<NAME>fileName</NAME>
	<VALUE>eigen.dat</VALUE>
</PARAM>
</PARAMETERS>

 

<STATES>
</STATES>
 

  

<INPUT_BUFFERS>
	<BUFFER>
		<TYPE>image_t</TYPE>
		<NAME>x</NAME>
	</BUFFER>
</INPUT_BUFFERS>
 


<DECLARATIONS> 
   	int i,j;
   	image_t img;
   	int fpixel;
   	
   	integer M ;
    integer N ;
    
    integer LDA;
    integer LDU ;
    integer LDVV;
    integer LWORK;
    integer INFO;

	int workLength;
    
    double* aa_P;
    double* uu_P;
    double* vv_P;
    double* work_P;
    double* eigenr_P;
    double* eigeni_P;
    
    FILE*  fp;
    
</DECLARATIONS> 
     

<INIT_CODE>
<![CDATA[ 

  /*
   * set the input buffer cell size to size of image
   */	
  SET_CELL_SIZE_IN(0,sizeof(image_t));

]]>
</INIT_CODE> 

 


<MAIN_CODE>
<![CDATA[ 

while(IT_IN(0)) {
       img=x(0);
   
   
       /*
        * allocate space for arrays
        */

       eigenr_P=(double*)calloc(img.width,sizeof(double));
       eigeni_P=(double*)calloc(img.width,sizeof(double));
       
       aa_P=(double*)calloc(img.width*img.height,sizeof(double));
       uu_P=(double*)calloc(img.width*img.height,sizeof(double));
       vv_P=(double*)calloc(img.width*img.height,sizeof(double));
	   if(img.width*img.height*2 < 8000)
		     workLength=8000;
	   else
		    workLength = img.width*img.height*2;

       work_P=(double*)calloc(workLength,sizeof(double));

       if(!aa_P || !uu_P || !vv_P || !work_P || !eigenr_P || !eigeni_P) {
            fprintf(stderr,"eigen: Could not allocate space\n"); 
            return(3);
       
       }
       
       M = img.width;
       N=img.height;
       
       if(M != N) {
            fprintf(stderr,"eigen: Matrix must be square\n"); 
            return(4);
       
       }
    
    
       LDA = M;
       LDU = M;
       LDVV = N;

	   LWORK=workLength;

       for ( i= 0; i< img.height; i++ ) {
	      for(j= 0; j< img.width; j++) {
		     aa_P[j*img.width+i]=img.image_PP[i][j];
	      }
       }


       /*
        * compute eigenvalues and eigenvectors
        */
       dgeev_( "V", "V", &M, aa_P, &LDA, eigenr_P, eigeni_P,uu_P, 
          &LDU, vv_P, &LDVV, work_P, &LWORK, &INFO);


       fprintf(stderr,"INFO=%d\n",INFO);
       for(i=0; i< img.height; i++) {
           printf("%d\t%f\t%f\n",i,eigenr_P[i],eigeni_P[i]);
       
       }


       /*
        * store eigenvalues
        */
        
       fp=fopen(fileName,"w");
       if(!fp) {
            fprintf(stderr,"eigen: could not open:%s to write\n",fileName); 
            return(5);
       }


       fprintf(stderr,"Storing computed eigenvalues in:%s\n",fileName); 


       for(i=0; i< img.height; i++) {
           fprintf(fp,"%d\t%f\t%f\n",i,eigenr_P[i],eigeni_P[i]);
       
       }
       
       fclose(fp);
       
       free(aa_P);
       free(uu_P);
       free(vv_P);
       free(work_P);
       free(eigenr_P);
       free(eigeni_P);
       

}
    
]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

]]>
</WRAPUP_CODE> 

</BLOCK> 


